/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks: SIN <sin@sapporoworks.ne.jp>
            http://homepage2.nifty.com/spw/
    He told me usage of RAW socket on Win32 !

 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "param.h"
#include "depend.h"
#include "defaultnic.h"

/*============================================================================
 Get default NIC
 [INPUT]
  none
 [RETURN]
  if default nic is not found, return NULL
  otherwise, name of default nic is returned
  ============================================================================
*/
char *getdefaultnic(void)
{
    static char         nicname[MAXLEN_NICNAME];
    WSADATA             WSAData;
    SOCKET              sock;
    char                buf[4096],*qnic_name;
    DWORD               bBytesReturned;
    SOCKET_ADDRESS_LIST *salist;
    SOCKADDR_IN         *nic_addr;
    
    WSAStartup(MAKEWORD(2,2),&WSAData);
     
    /* Create RAW socket */
    sock=WSASocket(AF_INET,SOCK_RAW,IPPROTO_IP,NULL,0,WSA_FLAG_OVERLAPPED);
    if(sock==INVALID_SOCKET){
        WSACleanup();
        return(NULL);
    }
     
    /* enum adapter list */
    if(WSAIoctl(sock,SIO_ADDRESS_LIST_QUERY,NULL,0,buf,4096,
                &bBytesReturned,NULL,NULL)==SOCKET_ERROR){
        closesocket(sock);
        WSACleanup();
        return(NULL);   
    }
    salist=(SOCKET_ADDRESS_LIST *)buf;
	if (salist->iAddressCount==0) return(NULL);

    nic_addr=(SOCKADDR_IN *)salist->Address[0].lpSockaddr;
	if (nic_addr==NULL) return(NULL);

    qnic_name=inet_ntoa(nic_addr->sin_addr);
    closesocket(sock);
    WSACleanup();
    if (strlen(qnic_name)>=sizeof(nicname)) return(NULL);
    strcpy(nicname,qnic_name);
    return(nicname);
}

/*============================================================================
 Get NIC list
 [INPUT]
	niclist     : Pointer to store niclist
	szniclist   : Size of niclist buffer
 [RETURN]
	-1=Error
	-2=Too short to store niclist
	 0=Success
  ============================================================================
*/
int	getniclist(char *niclist,unsigned int szniclist)
{
    static char         nicname[MAXLEN_NICNAME];
    WSADATA             WSAData;
    SOCKET              sock;
    char                buf[4096],*qnic_name;
    DWORD               bBytesReturned;
    SOCKET_ADDRESS_LIST *salist;
    SOCKADDR_IN         *nic_addr;
	int                 i;
    
    WSAStartup(MAKEWORD(2,2),&WSAData);
     
    /* Create RAW socket */
    sock=WSASocket(AF_INET,SOCK_RAW,IPPROTO_IP,NULL,0,WSA_FLAG_OVERLAPPED);
    if(sock==INVALID_SOCKET){
        WSACleanup();
        return(-1);
    }
     
    /* enum adapter list */
    if(WSAIoctl(sock,SIO_ADDRESS_LIST_QUERY,NULL,0,buf,4096,
                &bBytesReturned,NULL,NULL)==SOCKET_ERROR){
        closesocket(sock);
        WSACleanup();
        return(-1);   
    }
    salist=(SOCKET_ADDRESS_LIST *)buf;

	/* store to niclist */
	strcpy(niclist,"");
    for (i=0;i<salist->iAddressCount;i++){
        nic_addr  =(SOCKADDR_IN *)salist->Address[i].lpSockaddr;
        qnic_name =inet_ntoa(nic_addr->sin_addr);
        if (strlen(niclist)+strlen(qnic_name)+1 >= szniclist-1)
			return(-2);
		strcat(niclist,qnic_name);
		strcat(niclist,",");
    }
    return(0);
}
