/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 *****************************************************************************/

#include <ctype.h>
#include <string.h>
#include "depend.h"
#include "promisc.h"

/*============================================================================
 Set network interface to promiscus mode.
 [INPUT]
  nit_dev       : NIT device name
  nic_name      : Name of network interface  
  csize         : Chunk size (dummy)
 [RETURN]
  ERR_PROMISC_OPEN              : Can not open device 
  ERR_PROMISC_READ              : Can not read status 
  ERR_PROMISC_WRITE             : Can not write status 
  ERR_PROMISC_BIND              : Can not bind (IRIX only)
  ERR_PROMISC_SETSOCKOPT        : setsockopt failure (IRIX only) 
  ERR_PROMISC_TOOLONG_NITDEV    : Too long nit_dev
  ERR_PROMISC_TOOLONG_NICNAME   : Too long nic_name
  otherwise                     : device handle
  ============================================================================
*/
SOCKET setnic_promisc(char *nit_dev,char *nic_name,int *csize)
{
	int             sock;
	struct ifreq    f;

	if (strlen(nic_name) >= sizeof(f.ifr_name))
		return(ERR_PROMISC_TOOLONG_NICNAME);
	if ((sock=socket(PF_PACKET,SOCK_PACKET,htons(ETH_P_ALL)))<0)
		return(ERR_PROMISC_OPEN);
	strcpy(f.ifr_name, nic_name);
	if (ioctl(sock,SIOCGIFFLAGS,&f)<0)
		return(ERR_PROMISC_READ);
	f.ifr_flags |= IFF_PROMISC;
	if (ioctl(sock,SIOCSIFFLAGS,&f)<0)
		return(ERR_PROMISC_WRITE);
	return(sock);
}
