/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    Thanks : rocket <rocket_z@dial.104.net> helped me to develop this code !
    
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "depend.h"
#include "param.h"
#include "promisc.h"

#define MAXBUFSIZE	4096

/*============================================================================
 Set network interface to promiscus mode.
 [INPUT]
  nit_dev       : NIT device name
  nic_name      : Name of network interface  
  csize         : Chunk size (dummy)
 [RETURN]
  ERR_PROMISC_OPEN              : Can not open device 
  ERR_PROMISC_WRITE             : Attach/Promiscus error
  ERR_PROMISC_READ              : Can not read status 
  ERR_PROMISC_BIND              : Can not bind (IRIX only)
  ERR_PROMISC_SETSOCKOPT        : setsockopt failure (IRIX only) 
  ERR_PROMISC_TOOLONG_NITDEV    : Too long nit_dev
  ERR_PROMISC_TOOLONG_NICNAME   : Too long nic_name
  otherwise                     : device handle
  ============================================================================
*/
SOCKET setnic_promisc(char *nit_dev,char *nic_name,int *csize)
{
	int                     ppa,sock,i,flg;
	long                    buf[MAXBUFSIZE];
	char                    nicname[MAXLEN_NICNAME];
	union DL_primitives     *dlp;
	dl_attach_req_t         req_atth;
	dl_promiscon_req_t      req_prms;
	dl_bind_req_t           req_bind;
	struct strioctl         sc;
	struct strbuf	        ctl;
	
	/* Get device name and PPA from nic_name */
	strncpy(nicname,nic_name,MAXLEN_NICNAME);
	nicname[MAXLEN_NICNAME-1]='\0';
	for (i=strlen(nicname)-1;i>=0;i--)
		if (!isdigit((int)nicname[i])) break;
	ppa=atoi(nicname+i+1);
	nicname[i+1]='\0';

	/* Open device */
	if((sock=open(nicname,O_RDWR))< 0) return(ERR_PROMISC_OPEN);

	/* Requests that DLS provider associate a physical PPA with a stream */
	req_atth.dl_primitive      = DL_ATTACH_REQ;
        req_atth.dl_ppa            = ppa;
	ctl.len                    = sizeof(dl_attach_req_t);
	ctl.buf                    = (void *)&req_atth;
	ctl.maxlen                 = 0;
	if (putmsg(sock,&ctl,NULL,0)<0) return(ERR_PROMISC_WRITE);
	/* Wait ACK */
	ctl.len                    = 0;
	ctl.buf                    = (void *)buf;
	ctl.maxlen                 = MAXBUFSIZE;
	flg                        = MSG_HIPRI;
	dlp = (union DL_primitives *) ctl.buf;
	if (getmsg(sock,&ctl,NULL,&flg)<0 || dlp->dl_primitive!=DL_OK_ACK)
		return(ERR_PROMISC_WRITE);

	/* Requests that DLS provider enable promiscuous mode */
	req_prms.dl_primitive      = DL_PROMISCON_REQ;
	req_prms.dl_level          = DL_PROMISC_PHYS;
	ctl.len                    = sizeof(dl_promiscon_req_t);
	ctl.buf                    = (void *)&req_prms;
	ctl.maxlen                 = 0;
	flg                        = 0;
	if (putmsg(sock,&ctl,NULL,flg)<0) return(ERR_PROMISC_WRITE);
	/* Wait ACK */
	ctl.len                    = 0;
	ctl.buf                    = (void *)buf;
	ctl.maxlen                 = MAXBUFSIZE;
	flg                        = MSG_HIPRI;
	dlp = (union DL_primitives *)ctl.buf;
	if (getmsg(sock,&ctl,NULL,&flg)<0 || dlp->dl_primitive!=DL_OK_ACK)
		return(ERR_PROMISC_WRITE);

	/* Requests that the DLS provider bind a DLSAP to a stream */
	memset(&req_bind,0,sizeof(dl_bind_req_t));
	req_bind.dl_primitive      = DL_BIND_REQ;
	req_bind.dl_sap            = FLAME_TYPE;
	req_bind.dl_service_mode   = DL_CLDLS;
	ctl.len                    = sizeof(dl_bind_req_t);
	ctl.buf                    = (void *)&req_bind;
	ctl.maxlen                 = 0;
	if (putmsg(sock,&ctl,NULL,0)<0) return(ERR_PROMISC_BIND);
	/* Wait BIND_ACK */
	ctl.len                    = 0;
	ctl.buf                    = (void *)buf;
	ctl.maxlen                 = MAXBUFSIZE;
	flg                        = MSG_HIPRI;
	dlp = (union DL_primitives *)ctl.buf;
	if (getmsg(sock,&ctl,NULL,&flg)<0 || dlp->dl_primitive!=DL_BIND_ACK)
		return(ERR_PROMISC_BIND);

	/* Set socket option */
	sc.ic_cmd                  = DLIOCRAW;
	sc.ic_timout               = -1;
	sc.ic_len                  = 0;
	sc.ic_dp                   = NULL;
	if (ioctl(sock,I_STR,&sc)<0 || ioctl(sock,I_FLUSH,FLUSHR)<0)
		return(ERR_PROMISC_SETSOCKOPT);
		
	return(sock);
}
